********************************************************************************
*                                                                              *
*          SAS Program to Score the TIMSS Advanced 2015 Physics Items          *
*                        TIMSS Advanced 2015 User Guide                        *
*                                                                              *
********************************************************************************.

SET MPRINT=ON .

DEFINE SCOREIT (TYPE  = !CHAREND('/') /
                ITEM  = !CHAREND('/') /
                RIGHT = !CHAREND('/') /
                NR    = !CHAREND('/') /
                NA    = !CHAREND('/') /
                OM    = !CHAREND('/') /
                OTHER = !CHAREND('/') ) .

  * Code for multiple-choice items .

  !IF (!UPCASE(!TYPE)=MC) !THEN
    !DO !I !IN (!ITEM) .
      RECODE !I (!RIGHT     = 1      ) /* Item Key         */
                (!NR        = 0      ) /* Not Reached      */
                (!NA        = SYSMIS ) /* Not Administered */
                (!OM        = 0      ) /* Omitted          */
                (!OTHER     = 0      ) /* Other Missing    */
                (ELSE       = 0      ) .
    !DOEND .
  !IFEND .

  * Code for constructed-response items .

  !IF (!UPCASE(!TYPE)=CR) !THEN
    !DO !I !IN (!ITEM) .
      RECODE !I (20 THRU 29 = 2      ) /* Full Credit      */
                (10 THRU 19 = 1      ) /* Partial Credit   */
                (70 THRU 79 = 0      ) /* No Credit        */
                (!NR        = 0      ) /* Not Reached      */
                (!NA        = SYSMIS ) /* Not Administered */
                (!OM        = 0      ) /* Omitted          */
                (!OTHER     = 0      ) /* Other Missing    */
                (ELSE       = 0      ) .
    !DOEND .
  !IFEND .

!ENDDEFINE .

DEFINE DOIT (INDIR  = !CHAREND('/') /
             OUTDIR = !CHAREND('/') /
             INFILE = !CHAREND('/') ) .

  * Get the student achievement data .

  GET FILE = !QUOTE(!CONCAT(!INDIR,'\',!INFILE,'.SAV')) .

  * Score multiple-choice items with A key .

  SCOREIT TYPE = MC / ITEM = PA33115   PA23038   PA33078   PA33088   PA33116   PA23039   PA23042
                             PA33083   PA33002   PA33095B  PA33066A  PA33109   PA33111B  /
          RIGHT = 1 / NR = 6 / NA = SYSMIS / OM = 9 / OTHER = 7 .

  * Score multiple-choice items with B key .

  SCOREIT TYPE = MC / ITEM = PA13011   PA13014   PA13019   PA33102AA PA33121   PA23071   PA23148
                             PA33120   PA23064   PA33086   PA33114   PA33065   PA33040   PA33110
                             PA33063   /
          RIGHT = 2 / NR = 6 / NA = SYSMIS / OM = 9 / OTHER = 7 .

  * Score multiple-choice items with C key .

  SCOREIT TYPE = MC / ITEM = PA13012   PA13013   PA13015   PA13018   PA33102AB PA33101A  PA23104
                             PA33057   PA33012   PA23048   PA23085   PA33059   PA33015   PA33028B
                             PA33054A  PA33066B  PA33099   PA33003   PA33094B  /
          RIGHT = 3 / NR = 6 / NA = SYSMIS / OM = 9 / OTHER = 7 .

  * Score multiple-choice items with D key .

  SCOREIT TYPE = MC / ITEM = PA13016   PA13017   PA33061A  PA33044   PA23029   PA33079A  PA33019
                             PA33009B  PA33098   PA33118   PA33029   PA33081   PA33045   /
          RIGHT = 4 / NR = 6 / NA = SYSMIS / OM = 9 / OTHER = 7 .

  * Score multiple-choice items with E key .

  SCOREIT TYPE = MC / ITEM = PA33102AC /
          RIGHT = 5 / NR = 6 / NA = SYSMIS / OM = 9 / OTHER = 7 .

  * Score Constructed-Response Items .

  SCOREIT TYPE = CR / ITEM = PA33061B  PA33004   PA33075   PA33102A  PA33102B  PA33005   PA33101B
                             PA23146   PA23041   PA23053   PA23119   PA23088   PA23066   PA33058
                             PA33047   PA33079B  PA33070   PA33011   PA23035   PA23012   PA23051
                             PA23130   PA23086   PA33073   PA33035   PA33119   PA33046   PA33069
                             PA33080   PA33009A  PA33028A  PA33054B  PA33095A  PA33090   PA33064A
                             PA33064B  PA33097   PA33008   PA33008A  PA33008B  PA33008C  PA33072
                             PA33077   PA33111A  PA33094A  PA33094C  /
          RIGHT = 0 / NR = 96 / NA = SYSMIS / OM = 99 / OTHER = 90 .

  EXECUTE .

  * Save the student data .

  SAVE OUTFILE = !QUOTE(!CONCAT(!OUTDIR,'\',!INFILE,'_SCR.SAV')) .

  NEW FILE .

!ENDDEFINE .

DOIT INDIR  = <Specify location of input SPSS data file using format "x:\xxx"> /
     OUTDIR = <Specify location of output SPSS data file using format "x:\xxx"> /
     INFILE = <Specify name of merged SPSS student achievement data file> .
